/**
 * 
 */
package org.gcube.portlets.user.td.unionwizardwidget.client;

import org.gcube.portlets.user.td.gwtservice.shared.exception.TDGWTSessionExpiredException;
import org.gcube.portlets.user.td.gwtservice.shared.tr.union.UnionSession;
import org.gcube.portlets.user.td.unionwizardwidget.client.progress.UnionProgressBarUpdater;
import org.gcube.portlets.user.td.unionwizardwidget.client.progress.UnionProgressListener;
import org.gcube.portlets.user.td.unionwizardwidget.client.progress.UnionProgressUpdater;
import org.gcube.portlets.user.td.widgetcommonevent.client.event.SessionExpiredEvent;
import org.gcube.portlets.user.td.widgetcommonevent.client.type.SessionExpiredType;
import org.gcube.portlets.user.td.widgetcommonevent.shared.TRId;
import org.gcube.portlets.user.td.wizardwidget.client.WizardCard;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.ui.FlexTable;
import com.sencha.gxt.core.client.util.Margins;
import com.sencha.gxt.widget.core.client.FramedPanel;
import com.sencha.gxt.widget.core.client.ProgressBar;
import com.sencha.gxt.widget.core.client.box.AlertMessageBox;
import com.sencha.gxt.widget.core.client.container.BoxLayoutContainer.BoxLayoutData;
import com.sencha.gxt.widget.core.client.container.VBoxLayoutContainer;
import com.sencha.gxt.widget.core.client.container.VBoxLayoutContainer.VBoxLayoutAlign;
import com.sencha.gxt.widget.core.client.event.HideEvent;
import com.sencha.gxt.widget.core.client.event.HideEvent.HideHandler;



/**
 * 
 * @author "Giancarlo Panichi" <a
 *         href="mailto:g.panichi@isti.cnr.it">g.panichi@isti.cnr.it</a>
 * 
 */
public class UnionOperationInProgressCard extends WizardCard implements
		UnionProgressListener {

	public static final int STATUS_POLLING_DELAY = 1000;
	protected UnionOperationInProgressCard thisCard;
	protected UnionSession unionSession;
	protected UnionProgressUpdater progressUpdater;

	public UnionOperationInProgressCard(final UnionSession unionSession) {
		super("Operation In Progress", "");

		this.unionSession = unionSession;
		thisCard = this;

		VBoxLayoutContainer operationInProgressPanel = new VBoxLayoutContainer();
		operationInProgressPanel.setVBoxLayoutAlign(VBoxLayoutAlign.CENTER);

		final FlexTable description = new FlexTable();
		description.setCellSpacing(10);
		description.setCellPadding(4);
		description.setBorderWidth(0);

		description.setHTML(0, 0,
				"<span style=\"font-weight:bold;\";>Union with: </span>");
		description.setText(0, 1, unionSession.getUnionTabularResource().getName());
		description.setHTML(1, 0,
				"<span style=\"font-weight:bold;\";>Type: </span>");
		description.setText(1, 1, unionSession.getUnionTabularResource().getTabularResourceType());

		description.setHTML(2, 0,
				"<span style=\"font-weight:bold;\";>Owner: </span>");
		description.setText(2, 1, unionSession.getUnionTabularResource().getOwnerLogin());

		FramedPanel summary = new FramedPanel();
		summary.setHeadingText("Union Summary");
		summary.setWidth(400);
		summary.add(description);
		operationInProgressPanel.add(summary, new BoxLayoutData(new Margins(20,
				5, 10, 5)));

		ProgressBar progressBar = new ProgressBar();
		operationInProgressPanel.add(progressBar, new BoxLayoutData(
				new Margins(10, 5, 10, 5)));

		progressUpdater = new UnionProgressUpdater();
		progressUpdater
				.addListener(new UnionProgressBarUpdater(progressBar));

		progressUpdater.addListener(this);

		setContent(operationInProgressPanel);

	}

	
	public void importCodelistMapping() {
		/*TDGWTServiceAsync.INSTANCE.startUnion(unionSession,
				new AsyncCallback<Void>() {

					public void onSuccess(Void result) {
						progressUpdater.scheduleRepeating(STATUS_POLLING_DELAY);
					}

					public void onFailure(Throwable caught) {
						if (caught instanceof TDGWTSessionExpiredException) {
							getEventBus()
									.fireEvent(
											new SessionExpiredEvent(
													SessionExpiredType.EXPIREDONSERVER));
						} else {
							showErrorAndHide(
									"Error in import codelist mapping",
									"An error occured in import codelist mapping: "
											+ caught.getLocalizedMessage(),
									caught.getStackTrace().toString(), caught);
						}
					}
				});
				*/
	}

	@Override
	public void setup() {
		getWizardWindow().setEnableBackButton(false);
		setBackButtonVisible(false);
		setNextButtonVisible(false);
		getWizardWindow().setEnableNextButton(false);
		getWizardWindow().setNextButtonToFinish();
		importCodelistMapping();
	}

	public void operationInitializing() {

	}

	public void operationUpdate(float elaborated) {

	}

	public void operationComplete(final TRId trId) {
	
		Command sayComplete = new Command() {
			public void execute() {
				try {
					getWizardWindow().close(false);
					Log.info("fire Complete: "+ trId);
	
					getWizardWindow().fireCompleted(trId);

				} catch (Exception e) {
					Log.error("fire Complete :" + e.getLocalizedMessage());
				}
			}
		};

		getWizardWindow().setNextButtonCommand(sayComplete);

		setNextButtonVisible(true);
		getWizardWindow().setEnableNextButton(true);
	}

	public void operationFailed(Throwable caught, String reason,
			String failureDetails) {
		if (caught instanceof TDGWTSessionExpiredException) {
			getEventBus()
					.fireEvent(
							new SessionExpiredEvent(
									SessionExpiredType.EXPIREDONSERVER));
		} else {
			AlertMessageBox d = new AlertMessageBox("Error in Codelist Mapping Import",
					reason);
			d.addHideHandler(new HideHandler() {
				public void onHide(HideEvent event) {
				}
			});
			d.show();
		}
	}

	@Override
	public void operationStopped(final TRId trId, String reason, String details) {
	
		Command sayComplete = new Command() {
			public void execute() {
				try {
					getWizardWindow().close(false);
					Log.info("fire Complete: tabular resource " + trId.getId());
					Log.info("fire Complete: tableId " + trId.getTableId());

					getWizardWindow().fireCompleted(trId);

				} catch (Exception e) {
					Log.error("fire Complete :" + e.getLocalizedMessage());
				}
			}
		};

		getWizardWindow().setNextButtonCommand(sayComplete);

		setNextButtonVisible(true);
		getWizardWindow().setEnableNextButton(true);

	}

	@Override
	public void operationGeneratingView() {
		// TODO Auto-generated method stub

	}

	@Override
	public void operationValidate(float elaborated) {
		// TODO Auto-generated method stub

	}

}
